package com.sd.business.service.production.impl;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.ruoyi.common.constant.StatusConstant;
import com.ruoyi.common.core.domain.BaseIdPo;
import com.ruoyi.common.utils.wrapper.IWrapper;
import com.sd.business.entity.bom.po.BomSpec;
import com.sd.business.entity.department.po.Department;
import com.sd.business.entity.order.enums.OrderStatusEnum;
import com.sd.business.entity.order.po.OrderInfo;
import com.sd.business.entity.order.po.OrderSku;
import com.sd.business.entity.production.dto.StockPreparationDto;
import com.sd.business.entity.production.vo.StockPreparationVo;
import com.sd.business.entity.sku.po.SkuSpec;
import com.sd.business.mapper.production.StockPreparationMapper;
import com.sd.business.service.order.OrderService;
import com.sd.business.service.order.OrderSkuService;
import com.sd.business.service.production.StockPreparationService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

@Service
public class StockPreparationServiceImpl implements StockPreparationService {

    @Resource
    private StockPreparationMapper stockPreparationMapper;

    @Autowired
    private OrderSkuService orderSkuService;

    @Autowired
    private OrderService orderService;

    @Override
    public List<StockPreparationVo> uncompletedList(StockPreparationDto dto) {
        IWrapper<StockPreparationVo> wrapper = createWrapper(dto);
        wrapper.eq("oi", OrderInfo::getStatus, OrderStatusEnum.STOCK_PREPARATION.getKey());
        wrapper.eq("os", OrderSku::getStockPreparationStatus, StatusConstant.NO);
        return stockPreparationMapper.uncompletedList(wrapper);
    }

    @Override
    public Page<StockPreparationVo> completedPage(StockPreparationDto dto) {
        IWrapper<StockPreparationVo> wrapper = createWrapper(dto);
        wrapper.eq("os", OrderSku::getStockPreparationStatus, StatusConstant.YES);
        wrapper.orderByDesc("os", OrderSku::getStockPreparationTime);
        wrapper.orderByAsc("os", OrderSku::getId);
        return stockPreparationMapper.completedPage(dto.getPage(), wrapper);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public synchronized void submit(List<Long> orderSkuIdList) {

        // 更爱状态为已备料
        List<OrderSku> orderSkuList = orderSkuService.list(q -> q
                .select(BaseIdPo::getId, OrderSku::getStockPreparationStatus, OrderSku::getOrderId)
                .eq(OrderSku::getStockPreparationStatus, StatusConstant.NO)
                .in(BaseIdPo::getId, orderSkuIdList));
        Date stockPreparationTime = new Date();
        for (OrderSku orderSku : orderSkuList) {
            orderSku.setStockPreparationTime(stockPreparationTime);
            orderSku.setStockPreparationStatus(StatusConstant.YES);
        }
        orderSkuService.updateBatchById(orderSkuList);

        List<Long> orderIdList = orderSkuList.stream().map(OrderSku::getOrderId).distinct().collect(Collectors.toList());
        List<OrderSku> list = orderSkuService.list(q -> q.in(OrderSku::getOrderId, orderIdList));
        Map<Long, List<OrderSku>> map = list.stream().collect(Collectors.groupingBy(OrderSku::getOrderId));

        List<OrderInfo> orderInfoList = new ArrayList<>();

        for (List<OrderSku> tempOrderSku : map.values()) {
            // 订单中未备料数量
            long count = tempOrderSku.stream()
                    .map(OrderSku::getStockPreparationStatus)
                    .filter(item -> item.equals(StatusConstant.NO))
                    .count();

            if (count != 0) {
                continue;
            }

            // 订单状态改为 生产中
            OrderInfo orderInfo = new OrderInfo();
            orderInfo.setId(tempOrderSku.get(0).getOrderId());
            orderInfo.setStatus(OrderStatusEnum.REJECT.getKey());
            orderInfoList.add(orderInfo);

            // TODO 生成生产任务和生产工单

            // TODO 出库

        }

        if (orderInfoList.size() > 0) {
            orderService.updateBatchById(orderInfoList);
        }

    }

    private IWrapper<StockPreparationVo> createWrapper(StockPreparationDto dto) {
        IWrapper<StockPreparationVo> wrapper = IWrapper.getWrapper();

        wrapper.eq("oi", OrderInfo::getCode, dto.getOrderCode());
        wrapper.eq("oi", OrderInfo::getWlnCode, dto.getOrderWlnCode());

        wrapper.eq("bs", BomSpec::getCode, dto.getBomSpecCode());
        wrapper.eq("bs", BomSpec::getName, dto.getBomSpecName());

        wrapper.eq("d", Department::getId, dto.getDepartmentId());

        wrapper.eq("ss", SkuSpec::getCode, dto.getSkuSpecCode());
        wrapper.eq("ss", SkuSpec::getName, dto.getSkuSpecName());

        wrapper.eq("bs", BomSpec::getWidth, dto.getWidth());

        return wrapper;
    }

}
